/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.person.service;

import org.springblade.core.mp.base.BaseService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.core.tool.api.R;
import org.springblade.person.dto.HouseUserDTO;
import org.springblade.person.entity.HouseUser;
import org.springblade.person.vo.HouseUserVO;

/**
 *  服务类
 *
 * @author BladeX
 * @since 2021-04-14
 */
public interface HouseUserService extends BaseService<HouseUser> {

	/**
	 * 自定义分页
	 *
	 * @param page
	 * @param basicInfo
	 * @return
	 */
	IPage<HouseUserVO> selectBasicInfoPage(IPage<HouseUserVO> page, HouseUserVO basicInfo);

    boolean deleteUserAndRelationship(HouseUserDTO userDto);

	Integer houseUserStat(String beginDate, String endDate);

	R submit(HouseUserDTO houseUserDTO);

	void submitUserTags(HouseUser houseUser);

	/**
		* 判断是否存在相同用户
	 * @param idCard 身份证
	 * @param roomId 房间id
	 * @return true  存在
	 * 		   false  不存在
	 */
	Boolean judgeHaveHouseUser(String idCard, Long roomId);
}
